{*********************************************************}
{*                                                       *}
{*  Turbo Vision Resource WorkShop Custom controls DLL   *}
{*                                                       *}
{*  Copyright (c) 1995-1996 Balzs Scheidler             *}
{*                                                       *}
{*********************************************************}

library RWCst;
{$R+,I-}

{$D TVRW custom controls DLL.}

uses
  Objects, TinyStrm, TxtStrm, DrvUtils, ExpView, MsgBox;

const
  idConsts = 60400;

type
  TConstInputData = record
    MaxLen: Longint;
    ValidMin, ValidMax: Longint;
  end;

  PConstInput = ^TConstInput;
  TConstInput = Object(TView)
    MaxLen,
    CurPos,
    FirstPos,
    SelStart,
    SelEnd: Integer;
    ValidType: Word;
    ValidOpt: Word;
    ValidMin, ValidMax: Word;
    constructor Init(ARemoteSelf: PExtView; var Bounds: TRect);
    constructor Load(ARemoteSelf: PExtView; var S: TStream);
    procedure Store(var S: TStream);
    function GetPalette: PPalette; virtual;
    procedure Draw; virtual;
    procedure SizeLimits(var Min, Max: TPoint); virtual;
  end;

constructor TConstInput.Init(ARemoteSelf: PExtView; var Bounds: TRect);
begin
  Bounds.Assign(Bounds.A.X, Bounds.A.Y, Bounds.A.X + 10, Bounds.A.Y + 1);
  inherited Init(ARemoteSelf, Bounds);
  MaxLen := 30;
  CurPos := 1;
  FirstPos := 1;
  SelStart := 1;
  SelEnd := 1;
  RemoteSelf^.Options := RemoteSelf^.Options or ofVersion20;
  ValidType := idConsts + 6;
  ValidOpt := 2;  {votransfer}
  ValidMin := 0;
  ValidMax := $FFFF;
end;

constructor TConstInput.Load(ARemoteSelf: PExtView; var S: TStream);
begin
  inherited Load(ARemoteSelf, S);
  S.Read(MaxLen, SizeOf(Integer)*5);
  DisposeStr(S.ReadStr);
  S.Read(ValidType, SizeOf(ValidType));
  if ValidType = idConsts+6 then begin { Read validator object }
    S.Read(ValidOpt, 2);
    S.Read(ValidMin, 4);
  end;
end;

procedure TConstInput.Store(var S: TStream);
var
  L: Byte;
begin
  inherited Store(S);
  S.Write(MaxLen, SizeOf(Integer)*5);
  L := 0;
  S.Write(L, 1);    { string = '' }
  if ValidType = idConsts+6 then begin { Write the validator object }
    S.Write(ValidType, 2);
    S.Write(ValidOpt, 2);
    S.Write(ValidMin, 4);
  end;
end;

function TConstInput.GetPalette: PPalette;
const
  P: String[1] = #19;
begin
  GetPalette := @P;
end;

procedure TConstInput.Draw;
var
  B: TDrawBuffer;
begin
  inherited Draw;
  WriteStr(1, 0, 'ConstInp', 1);
end;

procedure TConstInput.SizeLimits(var Min, Max: TPoint);
begin
  Min.X := 3;
  Min.Y := 1;
  Max.Y := 1;
end;

{ Allocate function. Such function is needed for all exported controls }
function ConstInput_Allocate: Pointer; export;
begin
  ConstInput_Allocate := New(PConstInput);
end;

{ Free function. TVRW will care about calling first ConstInput_Done and then
  ConstInput_Free, so DO NOT use Dispose(P, Done) here. The same applies to the
  allocate function. }
procedure ConstInput_Free(P: PConstInput); export;
begin
  Dispose(P);
end;

function ConstInput_Init(Self: PConstInput; RemoteSelf: PExtView; var Bounds: TRect): Boolean; export;
begin
  ConstInput_Init := Self^.Init(RemoteSelf, Bounds);
end;

procedure ConstInput_Done(Self: PConstInput); export;
begin
  Self^.Done;
end;

function ConstInput_Load(Self: PConstInput; RemoteSelf: PExtView; MemBlock: Pointer; Len: Word): Boolean; export;
var
  S: PStream;
begin
  S := New(PTinyStream, Init(MemBlock, Len));
  ConstInput_Load := Self^.Load(RemoteSelf, S^);
  Dispose(S, Done);
end;

procedure ConstInput_Store(Self: PConstInput; MemBlock: Pointer; MaxLen: Word; var Len: Word); export;
var
  S: PStream;
begin
  S := New(PTinyStream, Init(MemBlock, MaxLen));
  Self^.Store(S^);
  Len := S^.GetPos;
  Dispose(S, Done);
end;

{ This procedure is used to build the Transfer structure of the settings dialog. }
{ TConstInput has only one field }
procedure ConstInput_CtrlToData(Self: PConstInput; Consts: PCollection; var Rec: TConstInputData); export;
begin
  Rec.MaxLen := Self^.MaxLen;
  Rec.ValidMin := Self^.ValidMin;
  Rec.ValidMax := Self^.ValidMax;
end;

{ The opposite of the above }
{ The consts parameter is used, when you use the Constant inputline feature of
  Turbo Vision Resource WorkShop. Just pass the Consts parameter in the transfer
  record }
procedure ConstInput_DataToCtrl(Self: PConstInput; var Rec: TConstInputData); export;
begin
  Self^.MaxLen := Rec.MaxLen;
  Self^.ValidMin := Rec.ValidMin;
  Self^.ValidMax := Rec.ValidMax;
end;

{ The sourcer procedure }
procedure ConstInput_WriteSource(Self: PConstInput; Lang: Word; ClassName, VarName,
                 LinkTo: PChar; MemBlock: PChar; MemSize: Word); export;
var
  Out: PTxtStream;
  Params: array [1..6] of Longint;
begin
  Out := New(PTxtStream, Init(New(PTinyStream, Init(MemBlock, MemSize))));
  case Lang of
    lnPascal:
      begin
        Params[1] := Longint(VarName);
        Params[2] := Longint(ClassName);
        Params[3] := Self^.MaxLen;
        Params[4] := Longint(VarName);
        Params[5] := Self^.ValidMin;
        Params[6] := Self^.ValidMax;
        Out^.WrParamText('%z := New(P%z, Init(R, %d));'#13+
	                 'PConstInput(%z)^.SetValidator(New(PConstValidator, Init(%d, %d)));'#13, Params);
      end;
    lnCPlusPlus:
      begin
        Out^.WrStr('/* C++ version of TConstInput is not supported. */');
      end;
  end;
  Out^.WrChar(#0, 1);
  Dispose(Out, Done);
end;

procedure ConstInput_WriteTransferFields(Self: PConstInput; Lang, TabNum: Word; MemBlock: PChar; MemSize: Word); export;
var
  Out: PTxtStream;
  Params: array [1..3] of Longint;
begin
  Out := New(PTxtStream, Init(New(PTinyStream, Init(MemBlock, MemSize))));
  Params[1] := TabNum;
  case Lang of
    lnPascal:
      begin
        Params[2] := TabNum;
        Params[3] := Self^.MaxLen;
        Out^.WrParamText('Consts%d: PConstCollection;'#13+
	                 'ConstInput%d: String[%d];'#13, Params);
      end;
    lnCPlusPlus:
      begin
        Out^.WrParamText('/* C++ version of TConstInput is not supported */'#13, Params);
      end;
  end;
  Out^.WrChar(#0, 1);
  Dispose(Out, Done);
end;

type
  TConstZoomData = record
    Hex: Boolean;
  end;

  PConstZoom = ^TConstZoom;
  TConstZoom = Object(TView)
    Link: PExtView;
    Hex: Boolean;
    constructor Init(ARemoteSelf: PExtView; var Bounds: TRect);
    constructor Load(ARemoteSelf: PExtView; var S: TStream);
    procedure Store(var S: TStream);
    procedure HandleEvent(var Event: TEvent); virtual;
    function GetPalette: PPalette; virtual;
    procedure Draw; virtual;
    procedure SizeLimits(var Min, Max: TPoint); virtual;
  end;

constructor TConstZoom.Init(ARemoteSelf: PExtView; var Bounds: TRect);
begin
  Bounds.Assign(Bounds.A.X, Bounds.A.Y, Bounds.A.X+3, Bounds.A.Y+1);
  inherited Init(ARemoteSelf, Bounds);
  Hex := False;
  Link := nil;
end;

constructor TConstZoom.Load(ARemoteSelf: PExtView; var S: TStream);
begin
  inherited Load(ARemoteSelf, S);
  GetPeerViewPtr(S, Link);
  S.Read(Hex, SizeOf(Hex));
end;

procedure TConstZoom.Store(var S: TStream);
begin
  inherited Store(S);
  PutPeerViewPtr(S, Link);
  S.Write(Hex, SizeOf(Hex));
end;

procedure TConstZoom.HandleEvent(var Event: TEvent);
begin
  case Event.What of
    evBroadCast:
      case Event.Command of
        cmIsLinkableTo:
          if Event.InfoPtr <> nil then begin
            { This view can be linked to a TConstInput, or a TInputLine }
            if (PString(Event.InfoPtr)^ = 'TConstInput') or
	       (PString(Event.InfoPtr)^ = 'TInputLine') then ClearEvent(Event);
          end;
        cmSetLinkedView:
          begin
            Link := Event.InfoPtr;
            ClearEvent(Event);
          end;
        cmGetLinkName:
          if Event.InfoPtr <> nil then begin
            PString(Event.InfoPtr)^ := '(TConstZoom)';
            ClearEvent(Event);
          end;
      end;
  end;
end;

function TConstZoom.GetPalette: PPalette;
const
  P: String[2] = #22#23;
begin
  GetPalette := @P;
end;

procedure TConstZoom.Draw;
var
  B: TDrawBuffer;
begin
  MoveCStr(B, #222'~*~'#221, GetColor($0102));
  WriteLine(0, 0, Size.X, Size.Y, B);
end;

procedure TConstZoom.SizeLimits(var Min, Max: TPoint);
begin
  Min.X := 3;
  Min.Y := 1;
  Max.Y := 1;
  Max.X := 3;
end;

{ Allocate function. Such function is needed for all exported controls }
function ConstZoom_Allocate: Pointer; export;
begin
  ConstZoom_Allocate := New(PConstZoom);
end;

{ Free function. TVRW will care about calling first ConstZoom_Done and then
  ConstZoom_Free, so DO NOT use Dispose(P, Done) here. The same applies to the
  allocate function. }
procedure ConstZoom_Free(P: PConstZoom); export;
begin
  Dispose(P);
end;

function ConstZoom_Init(Self: PConstZoom; RemoteSelf: PExtView; var Bounds: TRect): Boolean; export;
begin
  ConstZoom_Init := Self^.Init(RemoteSelf, Bounds);
end;

procedure ConstZoom_Done(Self: PConstZoom); export;
begin
  Self^.Done;
end;

function ConstZoom_Load(Self: PConstZoom; RemoteSelf: PExtView; MemBlock: Pointer; Len: Word): Boolean; export;
var
  S: PStream;
begin
  S := New(PTinyStream, Init(MemBlock, Len));
  ConstZoom_Load := Self^.Load(RemoteSelf, S^);
  Dispose(S, Done);
end;

procedure ConstZoom_Store(Self: PConstZoom; MemBlock: Pointer; MaxLen: Word; var Len: Word); export;
var
  S: PStream;
begin
  S := New(PTinyStream, Init(MemBlock, MaxLen));
  Self^.Store(S^);
  Len := S^.GetPos;
  Dispose(S, Done);
end;

procedure ConstZoom_CtrlToData(Self: PConstZoom; Consts: PCollection; var Rec: TConstZoomData); export;
begin
  Rec.Hex := Self^.Hex;
end;

procedure ConstZoom_DataToCtrl(Self: PConstZoom; var Rec: TConstZoomData); export;
begin
  Self^.Hex := Rec.Hex;
end;

procedure ConstZoom_WriteSource(Self: PConstZoom; Lang: Word; ClassName, VarName,
                 LinkTo: PChar; MemBlock: PChar; MemSize: Word); export;
var
  Out: PTxtStream;
  Params: array [1..6] of Longint;
  Tmp2: String[5];
const
  Tmp1: array [0..3] of Char = 'nil';
begin
  Out := New(PTxtStream, Init(New(PTinyStream, Init(MemBlock, MemSize))));
  case Lang of
    lnPascal:
      begin
        Params[1] := Longint(VarName);
        Params[2] := Longint(ClassName);
        if (LinkTo = nil) or (LinkTo^ = #0) then
	  Params[3] := Longint(@Tmp1)
	 else
	  Params[3] := Longint(LinkTo);
        Params[4] := Longint(@Tmp2);
        if Self^.Hex then Tmp2 := 'True' else Tmp2 := 'False';
        Out^.WrParamText('%z := New(P%z, Init(R, PInputLine(%z), %s));'#13, Params);
      end;
    lnCPlusPlus:
      begin
        Out^.WrStr('/* C++ version of TConstZoom is not supported. */');
      end;
  end;
  Out^.WrChar(#0, 1);
  Dispose(Out, Done);
end;

procedure ConstZoom_WriteTransferFields(Self: PConstZoom; Lang, TabNum: Word; MemBlock: PChar; MemSize: Word); export;
var
  Out: PTxtStream;
  Params: array [1..3] of Longint;
begin
  Out := New(PTxtStream, Init(New(PTinyStream, Init(MemBlock, MemSize))));
  Params[1] := TabNum;
  case Lang of
    lnPascal:
      begin
        Out^.WrParamText('Consts%d: PConstCollection;'#13, Params);
      end;
    lnCPlusPlus:
      begin
        Out^.WrParamText('/* C++ version of TConstZoom is not supported */'#13, Params);
      end;
  end;
  Out^.WrChar(#0, 1);
  Dispose(Out, Done);
end;

type
  PConstDisplay = ^TConstDisplay;
  TConstDisplay = Object(TView)
    Link: PExtView;
    constructor Init(ARemoteSelf: PExtView; var Bounds: TRect);
    constructor Load(ARemoteSelf: PExtView; var S: TStream);
    procedure Store(var S: TStream);
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure Draw; virtual;
    procedure SizeLimits(var Min, Max: TPoint); virtual;
  end;

constructor TConstDisplay.Init(ARemoteSelf: PExtView; var Bounds: TRect);
begin
  Bounds.Assign(Bounds.A.X, Bounds.A.Y, Bounds.A.X + 15, Bounds.A.Y + 1);
  inherited Init(ARemoteSelf, Bounds);
  RemoteSelf^.EventMask := RemoteSelf^.EventMask or evBroadCast;
end;

constructor TConstDisplay.Load(ARemoteSelf: PExtView; var S: TStream);
begin
  inherited Load(ARemoteSelf, S);
  GetPeerViewPtr(S, Link);
end;

procedure TConstDisplay.Store(var S: TStream);
begin
  inherited Store(S);
  PutPeerViewPtr(S, Link);
end;

procedure TConstDisplay.HandleEvent(var Event: TEvent);
var
  Err: Word;
begin
  if Event.What = evBroadCast then
    case Event.Command of
        cmIsLinkableTo:
          if Event.InfoPtr <> nil then begin
            { This view can be linked to a TConstInput, or a TInputLine }
            if (PString(Event.InfoPtr)^ = 'TConstInput') or
	       (PString(Event.InfoPtr)^ = 'TInputLine') then ClearEvent(Event);
          end;
        cmSetLinkedView:
          begin
            Link := Event.InfoPtr;
            ClearEvent(Event);
          end;
        cmGetLinkName:
          if Event.InfoPtr <> nil then begin
            PString(Event.InfoPtr)^ := '(TConstDisplay)';
            ClearEvent(Event);
          end;
    end;
end;

procedure TConstDisplay.Draw;
var
  B: TDrawBuffer;
begin
  MoveChar(B, ' ', GetColor(1), Size.X);
  MoveStr(B[1], 'ConstDisplay', GetColor(1));
  WriteLine(0, 0, Size.X, Size.Y, B);
end;

procedure TConstDisplay.SizeLimits(var Min, Max: TPoint);
begin
  Min.X := 3;
  Min.Y := 1;
  Max.Y := 1;
end;

function ConstDisplay_Allocate: Pointer; export;
begin
  ConstDisplay_Allocate := New(PConstDisplay);
end;

procedure ConstDisplay_Free(P: PConstDisplay); export;
begin
  Dispose(P);
end;

function ConstDisplay_Init(Self: PConstDisplay; RemoteSelf: PExtView; var Bounds: TRect): Boolean; export;
begin
  ConstDisplay_Init := Self^.Init(RemoteSelf, Bounds);
end;

procedure ConstDisplay_Done(Self: PConstDisplay); export;
begin
  Self^.Done;
end;

function ConstDisplay_Load(Self: PConstDisplay; RemoteSelf: PExtView; MemBlock: Pointer; Len: Word): Boolean; export;
var
  S: PStream;
begin
  S := New(PTinyStream, Init(MemBlock, Len));
  ConstDisplay_Load := Self^.Load(RemoteSelf, S^);
  Dispose(S, Done);
end;

procedure ConstDisplay_Store(Self: PConstDisplay; MemBlock: Pointer; MaxLen: Word; var Len: Word); export;
var
  S: PStream;
begin
  S := New(PTinyStream, Init(MemBlock, MaxLen));
  Self^.Store(S^);
  Len := S^.GetPos;
  Dispose(S, Done);
end;

procedure ConstDisplay_WriteSource(Self: PConstDisplay; Lang: Word; ClassName, VarName,
                 LinkTo: PChar; MemBlock: PChar; MemSize: Word); export;
var
  Out: PTxtStream;
  Params: array [1..6] of Longint;
const
  Tmp1: array [0..3] of Char = 'nil';
begin
  Out := New(PTxtStream, Init(New(PTinyStream, Init(MemBlock, MemSize))));
  case Lang of
    lnPascal:
      begin
        Params[1] := Longint(VarName);
        Params[2] := Longint(ClassName);
        if (LinkTo = nil) or (LinkTo^ = #0) then Params[3] := Longint(@Tmp1) else Params[3] := Longint(LinkTo);
        Out^.WrParamText('%z := New(P%z, Init(R, PInputLine(%z)));'#13, Params);
      end;
    lnCPlusPlus:
      begin
        Out^.WrStr('/* C++ version of TConstDisplay is not supported. */');
      end;
  end;
  Out^.WrChar(#0, 1);
  Dispose(Out, Done);
end;

procedure ConstDisplay_WriteTransferFields(Self: PConstDisplay; Lang, TabNum: Word; MemBlock: PChar; MemSize: Word); export;
var
  Out: PTxtStream;
  Params: array [1..3] of Longint;
begin
  Out := New(PTxtStream, Init(New(PTinyStream, Init(MemBlock, MemSize))));
  Params[1] := TabNum;
  case Lang of
    lnPascal:
      begin
        Out^.WrParamText('Consts%d: PConstCollection;'#13, Params);
      end;
    lnCPlusPlus:
      begin
        Out^.WrParamText('/* C++ version of TConstDisplay is not supported */'#13, Params);
      end;
  end;
  Out^.WrChar(#0, 1);
  Dispose(Out, Done);
end;

{ Data structure used when communicating with TVRW. }
const
  DConstInput: TControlData =
    (Size: SizeOf(TControlData);
     Id: 'TVRWCust_ConstInput';  { Has to be unique among all external controls,
                               that's why I included the DLL name and the class name }
     TypeId: 'ConstInput';
     Description: 'Constant Inputline';
     ObjType: idConsts+2;
     PrefixStr: 'CONSTINPUT');

  DConstZoom: TControlData =
    (Size: SizeOf(TControlData);
     Id: 'TVRWCust_ConstZoom';
     TypeId: 'ConstZoom';
     Description: 'Constant zoomer';
     ObjType: idConsts+3;
     PrefixStr: 'CONSTZOOM');

  DConstDisplay: TControlData =
    (Size: SizeOf(TControlData);
     Id: 'TVRWCust_ConstDisplay';
     TypeId: 'ConstDisplay';
     Description: 'Constant displayer';
     ObjType: idConsts+4;
     PrefixStr: 'ConstDisplay');

const
  ExportedControls: array [1..4] of PControlData =
    (@DConstInput, @DConstZoom, @DConstDisplay, nil);

var
  CurrCtrl: Integer;

{ Findfirst function. Resets search and returns the first available control }
function CtrlFindFirst: PControlData; export;
begin
  CurrCtrl := 2;
  CtrlFindFirst := ExportedControls[1];
end;

{ Findnext ... }
function CtrlFindNext: PControlData; export;
begin
  CtrlFindNext := ExportedControls[CurrCtrl];
  Inc(CurrCtrl);
end;


{ DLL initialization procedure. TVRW will not accept DLLs without this routine. }
procedure InitControlDLL(TVRWInstance: Word; var Description: String); export;
begin
  LinkProcedures(TVRWInstance);
  Description := 'TVRW Custom Control DLL.';
end;

{ deinitialization }
procedure DoneControlDLL; export;
begin
end;

{ About procedure. }
procedure AboutControlDLL; export;
var
  R: TRect;
begin
  R.Assign(15, 6, 65, 10);
  MessageBoxRect(R, ^C'TVRW Custom Control Library'^M^M+
                'Copyright (c) 1995-96 Balzs Scheidler',
                nil, mfInformation or mfOKButton);
end;

exports
  ConstInput_Allocate,
  ConstInput_Free,
  ConstInput_Init,
  ConstInput_Done,
  ConstInput_Load,
  ConstInput_Store,
  ConstInput_CtrlToData,
  ConstInput_DataToCtrl,
  ConstInput_WriteSource,
  ConstInput_WriteTransferFields,
  ConstZoom_Allocate,
  ConstZoom_Free,
  ConstZoom_Init,
  ConstZoom_Done,
  ConstZoom_Load,
  ConstZoom_Store,
  ConstZoom_CtrlToData,
  ConstZoom_DataToCtrl,
  ConstZoom_WriteSource,
  ConstZoom_WriteTransferFields,
  ConstDisplay_Allocate,
  ConstDisplay_Free,
  ConstDisplay_Init,
  ConstDisplay_Done,
  ConstDisplay_Load,
  ConstDisplay_Store,
  ConstDisplay_WriteSource,
  ConstDisplay_WriteTransferFields,
  ViewDraw,
  ViewGetPalette,
  ViewHandleEvent,
  ViewSizeLimits,
  ViewValid,
  ViewChangeBounds,
  ViewSetData,
  ViewGetData,
  ViewDataSize,
  InitControlDLL,
  DoneControlDLL,
  AboutControlDLL,
  CtrlFindFirst,
  CtrlFindNext;

begin
end.
