{*********************************************************}
{*                                                       *}
{*  Turbo Vision Resource WorkShop Demonstration DLL     *}
{*                                                       *}
{*  Copyright (c) 1995-1996 Balzs Scheidler             *}
{*                                                       *}
{*********************************************************}

library TestView;
{$R+,I-}

{$D TVRW sample custom control DLL.}

uses
  Objects, TinyStrm, TxtStrm, DrvUtils, ExpView, MsgBox;

type
  { Used as the Transfer structure of the MyView Settings dialog }
  TMyViewData = record
    Str: String;
  end;

  PMyView = ^TMyView;
  TMyView = Object(TView)
    Str: PString;
    Color: Byte;
    Link: PExtView;
    { The only difference in Init and Load is the ARemoteSelf parameter.
      You don't need to bother with this variable, just pass it to the
      inherited method. }
    constructor Init(ARemoteSelf: PExtView; var Bounds: TRect);
    constructor Load(ARemoteSelf: PExtView; var S: TStream);
    procedure Store(var S: TStream);
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure Draw; virtual;
  end;

constructor TMyView.Init(ARemoteSelf: PExtView; var Bounds: TRect);
begin
  Bounds.Assign(Bounds.A.X, Bounds.A.Y, Bounds.A.X + 45, Bounds.A.Y + 7);
  inherited Init(ARemoteSelf, Bounds);
  Str := NewStr('This is written from TESTVIEW.DLL');
  Color := 1;
end;

constructor TMyView.Load(ARemoteSelf: PExtView; var S: TStream);
begin
  inherited Load(ARemoteSelf, S);
  GetPeerViewPtr(S, Link);
  Str := S.ReadStr;
  Color := 1;
  if RemoteSelf^.Options and ofViewExtensions <> 0 then begin
    { Read the data you wrote the similar section in store }
  end;
end;

procedure TMyView.Store(var S: TStream);
begin
  inherited Store(S);
  PutPeerViewPtr(S, Link);
  S.WriteStr(Str);
  if RemoteSelf^.Options and ofViewExtensions <> 0 then begin
    { If you add some data to your object that is not normally stored by
      your object.... Such an extension would be for instance the
      string representation of HelpCtx }
  end;
end;

procedure TMyView.HandleEvent(var Event: TEvent);
begin
  inherited HandleEvent(Event);
  case Event.What of
    evBroadCast:
      case Event.Command of
        cmIsLinkableTo:
          if Event.InfoPtr <> nil then begin
            { This view can be linked to an Inputline }
            if PString(Event.InfoPtr)^ = 'TInputLine' then ClearEvent(Event);
          end;
        cmSetLinkedView:
          begin
            Link := Event.InfoPtr;
            DrawView;
            ClearEvent(Event);
          end;
        cmGetLinkName:
          if Event.InfoPtr <> nil then begin
            if Str <> nil then
              PString(Event.InfoPtr)^ := '(TMyView)'
             else
              PString(Event.InfoPtr)^ := '';
            ClearEvent(Event);
          end;
      end;
    evMouseDown:
      begin
        if Event.Buttons = mbLeftButton then
          Inc(Color)
         else
          Dec(Color);
        if Event.Double then Color := 1;
        DrawView;
        ClearEvent(Event);
      end;
  end;
end;

procedure TMyView.Draw;
var
  B: TDrawBuffer;
begin
  inherited Draw;
  if Str <> nil then begin
    MoveStr(B, Str^, GetColor(Color));
    WriteLine((Size.X - Length(Str^)) shr 1, Size.Y shr 1, Length(Str^), 1, B);
  end;
  if Link <> nil then begin
    WriteStr(1, 1, 'Linked.', Color);
   end
   else
    WriteStr(1, 1, 'Not linked.', Color);
end;

{ Allocate function. Such function is needed for all exported controls }
function MyView_Allocate: Pointer; export;
begin
  MyView_Allocate := New(PMyView);
end;

{ Free function. TVRW will care about calling first MyView_Done and then
  MyView_Free, so DO NOT use Dispose(P, Done) here. The same applies to the
  allocate function. }
procedure MyView_Free(P: PMyView); export;
begin
  Dispose(P);
end;

function MyView_Init(Self: PMyView; RemoteSelf: PExtView; var Bounds: TRect): Boolean; export;
begin
  MyView_Init := Self^.Init(RemoteSelf, Bounds);
end;

procedure MyView_Done(Self: PMyView); export;
begin
  Self^.Done;
end;

function MyView_Load(Self: PMyView; RemoteSelf: PExtView; MemBlock: Pointer; Len: Word): Boolean; export;
var
  S: PStream;
begin
  S := New(PTinyStream, Init(MemBlock, Len));
  MyView_Load := Self^.Load(RemoteSelf, S^);
  Dispose(S, Done);
end;

procedure MyView_Store(Self: PMyView; MemBlock: Pointer; MaxLen: Word; var Len: Word); export;
var
  S: PStream;
begin
  S := New(PTinyStream, Init(MemBlock, MaxLen));
  Self^.Store(S^);
  Len := S^.GetPos;
  Dispose(S, Done);
end;

{ This procedure is used to build the Transfer structure of the settings dialog. }
{ TMyView has only one field }
procedure MyView_CtrlToData(Self: PMyView; Consts: PCollection; var Rec: TMyViewData); export;
begin
  if Self^.Str <> nil then Rec.Str := Self^.Str^ else Rec.Str := '';
end;

{ The opposite of the above }
{ The consts parameter is used, when you use the Constant inputline feature of
  Turbo Vision Resource WorkShop. Just pass the Consts parameter in the transfer
  record }
procedure MyView_DataToCtrl(Self: PMyView; var Rec: TMyViewData); export;
begin
  DisposeStr(Self^.Str);
  Self^.Str := NewStr(Rec.Str);
end;

{ The sourcer procedure }
procedure MyView_WriteSource(Self: PMyView; Lang: Word; ClassName, VarName, 
                 LinkTo: PChar; MemBlock: PChar; MemSize: Word); export;
var
  Out: PTxtStream;
  Params: array [1..5] of Pointer;
const
  NilStr: array [0..4] of Char = 'nil';
  NullStr: array [0..5] of Char = 'NULL';
begin
  Out := New(PTxtStream, Init(New(PTinyStream, Init(MemBlock, MemSize))));
  { at this time the necessary variables had been set up, so we need to output
    only the init statement
    TVRW will care about the options, eventmask and state flags
    You should obey the following rules to produce the correct output:
      To indent to the following tab-stop insert a tab #9 character
      To insert a newline simply insert a #13 character. You don't need to care
         about the indentation in the next line
      The size of the memory block is about 16384 bytes, I think this should be
         enough for most views. }
  Params[1] := VarName;
  Params[2] := ClassName;
  Params[3] := Self^.Str;
  case Lang of
    lnPascal:
      begin
        if (LinkTo = nil) or (LinkTo^ = #0)  then Params[4] := @NilStr else Params[4] := LinkTo;
        Out^.WrParamText('%z := New(P%z, Init(R, ''%s'', PInputLine(%z)));'#13, Params);
      end;
    lnCPlusPlus:
      begin
        if (LinkTo = nil) or (LinkTo^ = #0) then Params[4] := @NullStr else Params[4] := LinkTo;
        Out^.WrParamText('%z = new T%z(r, "%s", (TInputLine *) %z);'#13, Params);
      end;
  end;
  Out^.WrChar(#0, 1);
  Dispose(Out, Done);
end;

procedure MyView_WriteTransferFields(Self: PMyView; Lang, TabNum: Word; MemBlock: PChar; MemSize: Word); export;
var
  Out: PTxtStream;
  Params: array [1..1] of Longint;
begin
  Out := New(PTxtStream, Init(New(PTinyStream, Init(MemBlock, MemSize))));
  Params[1] := TabNum;
  case Lang of
    lnPascal:
      begin
        Out^.WrParamText('{ object TMyView has no parameters. Tab #%d }'#13, Params);
      end;
    lnCPlusPlus:
      begin
        Out^.WrParamText('/* class TMyView has no parameters. Tab #%d */'#13, Params);
      end;
  end;
  Out^.WrChar(#0, 1);
  Dispose(Out, Done);
end;

{ Data structure used when communicating with TVRW. }
const
  DMyView: TControlData =
    (Size: SizeOf(TControlData);
     Id: 'TestView_MyView';  { Has to be unique among all external controls,
                               that's why I included the DLL name and the class name }
     TypeId: 'MyView';
     Description: 'Blue Textbox';
     ObjType: 20000;
     PrefixStr: 'MYVIEW');

const
  ExportedControls: array [1..2] of PControlData =
    (@DMyView, nil);

var
  CurrCtrl: Integer;

{ Findfirst function. Resets search and returns the first available control }
function CtrlFindFirst: PControlData; export;
begin
  CurrCtrl := 2;
  CtrlFindFirst := ExportedControls[1];
end;

{ Findnext ... }
function CtrlFindNext: PControlData; export;
begin
  CtrlFindNext := ExportedControls[CurrCtrl];
  CurrCtrl := CurrCtrl + 1;
end;


{ DLL initialization procedure. TVRW will not accept DLLs without this routine. }
procedure InitControlDLL(TVRWInstance: Word; var Description: String); export;
begin
  LinkProcedures(TVRWInstance);
  Description := 'Sample Control DLL.';
end;

{ deinitialization }
procedure DoneControlDLL; export;
begin
end;

{ About procedure. }
procedure AboutControlDLL; export;
begin
  MessageBox(^C'Sample Control Library'^M+
               'Copyright (c) 1995-96 Balzs Scheidler',
               nil, mfInformation or mfOKButton);
end;

exports
  MyView_Allocate,
  MyView_Free,
  MyView_Init,
  MyView_Done,
  MyView_Load,
  MyView_Store,
  MyView_CtrlToData,
  MyView_DataToCtrl,
  MyView_WriteSource,
  MyView_WriteTransferFields,
  ViewDraw,
  ViewGetPalette,
  ViewHandleEvent,
  ViewSizeLimits,
  ViewValid,
  ViewChangeBounds,
  ViewSetData,
  ViewGetData,
  ViewDataSize,
  InitControlDLL,
  DoneControlDLL,
  AboutControlDLL,
  CtrlFindFirst,
  CtrlFindNext;

begin
end.
